@@ -121,7 +121,7 @@ def test_aot_unwrap():
     d = item([{"a": "A"}, {"b": "B"}])
     unwrapped = d.unwrap()
     assert_is_ppo(unwrapped, list)
-    for du, _ in zip(unwrapped, d):
+    for du, _ in zip(unwrapped, d):  # noqa: B905
         assert_is_ppo(du, dict)
         for ku in du:
             vu = du[ku]
@@ -129,6 +129,19 @@ def test_aot_unwrap():
             assert_is_ppo(vu, str)
 
 
+def test_aot_set_item():
+    d = item(["A", {"b": "B"}, ["c", "D"]])
+    d[0] = "C"
+    assert isinstance(d[0], String)
+    assert d[0] == "C"
+    d[1]["b"] = "D"
+    assert isinstance(d[1], InlineTable)
+    assert d[1]["b"] == "D"
+    d[0] = ["c", "C"]
+    assert isinstance(d[0], Array)
+    assert d[0][1] == "C"
+
+
 def test_time_unwrap():
     t = time(3, 8, 14)
     elementary_test(item(t), time)
@@ -624,6 +637,10 @@ def test_integers_behave_like_ints():
     assert i == 33
     assert i.as_string() == "33"
 
+    i /= 2
+    assert i == 16.5
+    assert i.as_string() == "16.5"
+
     doc = parse("int = +34")
     doc["int"] += 1
 
@@ -685,7 +702,7 @@ def test_dates_behave_like_dates():
     assert i.as_string() == "2018-07-22"
 
     i += timedelta(days=1)
-    assert i == datetime(2018, 7, 23)
+    assert i == date(2018, 7, 23)
     assert i.as_string() == "2018-07-23"
 
     i -= timedelta(days=2)
@@ -693,7 +710,7 @@ def test_dates_behave_like_dates():
     assert i.as_string() == "2018-07-21"
 
     i = i.replace(year=2019)
-    assert i == datetime(2019, 7, 21)
+    assert i == date(2019, 7, 21)
     assert i.as_string() == "2019-07-21"
 
     doc = parse("dt = 2018-07-22 # Comment")
@@ -911,6 +928,8 @@ value = false
 """
 
     content = parse(s)
+    assert content["foo"]["value"] is False
+    assert isinstance(content["foo"].item("value"), Bool)
 
     assert {"foo": {"value": False}} == content
     assert {"value": False} == content["foo"]
@@ -965,3 +984,218 @@ def test_custom_encoders():
 
     assert api.dumps({"foo": decimal.Decimal("1.23")}) == "foo = 1.23\n"
     api.unregister_encoder(encode_decimal)
+
+
+def test_custom_encoders_with_parent_and_sort_keys():
+    """Test that custom encoders can receive _parent and _sort_keys parameters."""
+    import decimal
+
+    parent_captured = None
+    sort_keys_captured = None
+
+    @api.register_encoder
+    def encode_decimal_with_context(obj, _parent=None, _sort_keys=False):
+        nonlocal parent_captured, sort_keys_captured
+        if isinstance(obj, decimal.Decimal):
+            parent_captured = _parent
+            sort_keys_captured = _sort_keys
+            return api.float_(str(obj))
+        raise TypeError
+
+    # Test with default parameters
+    result = api.item(decimal.Decimal("1.23"))
+    assert result.as_string() == "1.23"
+    assert parent_captured is None
+    assert sort_keys_captured is False
+
+    # Test with custom parent and sort_keys
+    parent_captured = None
+    sort_keys_captured = None
+    table = api.table()
+    result = item(decimal.Decimal("4.56"), _parent=table, _sort_keys=True)
+    assert result.as_string() == "4.56"
+    assert parent_captured is table
+    assert sort_keys_captured is True
+
+    api.unregister_encoder(encode_decimal_with_context)
+
+
+def test_custom_encoders_backward_compatibility():
+    """Test that old-style custom encoders still work without modification."""
+    import decimal
+
+    @api.register_encoder
+    def encode_decimal_old_style(obj):
+        # Old style encoder - only accepts obj parameter
+        if isinstance(obj, decimal.Decimal):
+            return api.float_(str(obj))
+        raise TypeError
+
+    # Should work exactly as before
+    result = api.item(decimal.Decimal("2.34"))
+    assert result.as_string() == "2.34"
+
+    # Should work when called from item() with extra parameters
+    table = api.table()
+    result = item(decimal.Decimal("5.67"), _parent=table, _sort_keys=True)
+    assert result.as_string() == "5.67"
+
+    api.unregister_encoder(encode_decimal_old_style)
+
+
+def test_custom_encoders_with_kwargs():
+    """Test that custom encoders can use **kwargs to accept additional parameters."""
+    import decimal
+
+    kwargs_captured = None
+
+    @api.register_encoder
+    def encode_decimal_with_kwargs(obj, **kwargs):
+        nonlocal kwargs_captured
+        if isinstance(obj, decimal.Decimal):
+            kwargs_captured = kwargs
+            return api.float_(str(obj))
+        raise TypeError
+
+    # Test with parent and sort_keys passed as kwargs
+    table = api.table()
+    result = item(decimal.Decimal("7.89"), _parent=table, _sort_keys=True)
+    assert result.as_string() == "7.89"
+    assert kwargs_captured == {"_parent": table, "_sort_keys": True}
+
+    api.unregister_encoder(encode_decimal_with_kwargs)
+
+
+def test_custom_encoders_for_complex_objects():
+    """Test custom encoders that need to encode nested structures."""
+
+    class CustomDict:
+        def __init__(self, data):
+            self.data = data
+
+    @api.register_encoder
+    def encode_custom_dict(obj, _parent=None, _sort_keys=False):
+        if isinstance(obj, CustomDict):
+            # Create a table and use item() to convert nested values
+            table = api.table()
+            for key, value in obj.data.items():
+                # Pass along _parent and _sort_keys when converting nested values
+                table[key] = item(value, _parent=table, _sort_keys=_sort_keys)
+            return table
+        raise TypeError
+
+    # Test with nested structure
+    custom_obj = CustomDict({"a": 1, "b": {"c": 2, "d": 3}})
+    result = item(custom_obj, _sort_keys=True)
+
+    # Should properly format as a table with sorted keys
+    expected = """a = 1
+
+[b]
+c = 2
+d = 3
+"""
+    assert result.as_string() == expected
+
+    api.unregister_encoder(encode_custom_dict)
+
+
+def test_no_extra_minus_sign():
+    doc = parse("a = -1")
+    assert doc.as_string() == "a = -1"
+    doc["a"] *= -1
+    assert doc.as_string() == "a = +1"
+    doc["a"] *= -1
+    assert doc.as_string() == "a = -1"
+
+    doc = parse("a = -1.5")
+    assert doc.as_string() == "a = -1.5"
+    doc["a"] *= -1
+    assert doc.as_string() == "a = +1.5"
+    doc["a"] *= -1
+    assert doc.as_string() == "a = -1.5"
+
+
+def test_serialize_table_with_dotted_key():
+    child = api.table()
+    child.add(api.key(("b", "c")), 1)
+    parent = api.table()
+    parent.add("a", child)
+    assert parent.as_string() == "[a]\nb.c = 1\n"
+
+
+def test_not_showing_parent_header_for_super_table():
+    doc = api.document()
+
+    def add_table(parent, name):
+        parent.add(name, api.table())
+        return parent[name]
+
+    root = add_table(doc, "root")
+    add_table(root, "first")
+    add_table(root, "second")
+    assert doc.as_string() == "[root.first]\n\n[root.second]\n"
+
+
+def test_removal_of_arrayitem_with_extra_whitespace():
+    expected = 'x = [\n    "bar",\n]'
+    doc = parse('x = [\n    "foo" ,#spam\n    "bar",\n]')
+    x = doc["x"]
+    assert isinstance(x, Array)
+    x.remove("foo")
+    docstr = doc.as_string()
+    parse(docstr)
+    assert docstr == expected
+
+
+def test_badly_formatted_array_and_item_removal():
+    expected = """
+x = [
+    '0'#a
+,#b
+
+    # comment
+'1' #c
+    , #d
+# another comment
+'2'
+ # yet another comment
+,'3', # f
+ # comments here
+'4'#g
+#    comments there
+ ,'5' #h
+    #   comments everywhere
+ # so many comments
+,'6' ,
+      # you get a comment
+# you get a comment
+   '7' ,#j
+# everybody gets a comment!!!
+ "8"#c
+,"9",  \n  "10"
+]
+"""
+    assert expected == parse(expected).as_string()
+    for i in range(11):
+        doc = parse(expected)
+        x = doc["x"]
+        assert isinstance(x, Array)
+        x.remove(str(i))
+        parse(doc.as_string())
+
+
+def test_array_item_removal_newline_restore_next():
+    expected = "x = [\n  '0',\n    '2'\n]"
+
+    docstr = "x = [\n  '0',\n    '1','2'\n]"
+    doc = parse(docstr)
+    doc["x"].remove("1")
+    assert doc.as_string() == expected
+    parse(doc.as_string())
+
+    docstr = "x = [\n  '0',\n    '1',  '2'\n]"
+    doc = parse(docstr)
+    doc["x"].remove("1")
+    assert doc.as_string() == expected
+    parse(doc.as_string())
