@@ -1,9 +1,9 @@
-from typing import Collection
-from typing import Optional
+from __future__ import annotations
+
+from collections.abc import Collection
 
 
 class TOMLKitError(Exception):
-
     pass
 
 
@@ -14,7 +14,7 @@ class ParseError(ValueError, TOMLKitError):
     location within the line where the error was encountered.
     """
 
-    def __init__(self, line: int, col: int, message: Optional[str] = None) -> None:
+    def __init__(self, line: int, col: int, message: str | None = None) -> None:
         self._line = line
         self._col = col
 
@@ -115,7 +115,7 @@ class UnexpectedCharError(ParseError):
     """
 
     def __init__(self, line: int, col: int, char: str) -> None:
-        message = f"Unexpected character: {repr(char)}"
+        message = f"Unexpected character: {char!r}"
 
         super().__init__(line, col, message=message)
 
@@ -148,7 +148,7 @@ class InvalidCharInStringError(ParseError):
     """
 
     def __init__(self, line: int, col: int, char: str) -> None:
-        message = f"Invalid character {repr(char)} in string"
+        message = f"Invalid character {char!r} in string"
 
         super().__init__(line, col, message=message)
 
@@ -169,7 +169,7 @@ class InternalParserError(ParseError):
     An error that indicates a bug in the parser.
     """
 
-    def __init__(self, line: int, col: int, message: Optional[str] = None) -> None:
+    def __init__(self, line: int, col: int, message: str | None = None) -> None:
         msg = "Internal parser error"
         if message:
             msg += f" ({message})"
@@ -194,6 +194,7 @@ class KeyAlreadyPresent(TOMLKitError):
     """
 
     def __init__(self, key):
+        key = getattr(key, "key", key)
         message = f'Key "{key}" already exists.'
 
         super().__init__(message)
@@ -209,8 +210,9 @@ class InvalidControlChar(ParseError):
         display_code += hex(char)[2:]
 
         message = (
-            "Control characters (codes less than 0x1f and 0x7f) are not allowed in {}, "
-            "use {} instead".format(type, display_code)
+            "Control characters (codes less than 0x1f and 0x7f)"
+            f" are not allowed in {type}, "
+            f"use {display_code} instead"
         )
 
         super().__init__(line, col, message=message)
@@ -223,3 +225,10 @@ class InvalidStringError(ValueError, TOMLKitError):
             f"Invalid string: {delimiter}{repr_}{delimiter}. "
             f"The character sequences {invalid_sequences} are invalid."
         )
+
+
+class ConvertError(TypeError, ValueError, TOMLKitError):
+    """Raised when item() fails to convert a value.
+    It should be a TypeError, but due to historical reasons
+    it needs to subclass ValueError as well.
+    """
